package sun.security.provider.certpath;

import jdk.internal.event.EventHelper;
import java.util.Date;
import java.security.cert.PKIXRevocationChecker;
import java.security.cert.PKIXCertPathChecker;
import java.util.Collections;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.security.cert.PKIXReason;
import java.security.cert.TrustAnchor;
import java.io.IOException;
import java.security.cert.CertificateException;
import sun.security.x509.X509CertImpl;
import java.security.cert.X509Certificate;
import java.security.cert.PKIXCertPathValidatorResult;
import java.security.InvalidAlgorithmParameterException;
import java.security.cert.CertPathValidatorException;
import java.security.cert.CertPathValidatorResult;
import java.security.cert.CertPathParameters;
import java.security.cert.CertPath;
import java.security.cert.CertPathChecker;
import java.util.concurrent.atomic.AtomicLong;
import sun.security.util.Debug;
import java.security.cert.CertPathValidatorSpi;

public final class PKIXCertPathValidator extends CertPathValidatorSpi
{
    private static final Debug debug;
    private static final AtomicLong validationCounter;
    
    public PKIXCertPathValidator() {
    }
    
    @Override
    public CertPathChecker engineGetRevocationChecker() {
        return new RevocationChecker();
    }
    
    @Override
    public CertPathValidatorResult engineValidate(CertPath certPath, CertPathParameters certPathParameters) throws CertPathValidatorException, InvalidAlgorithmParameterException {
        return validate(PKIX.checkParams(certPath, certPathParameters));
    }
    
    private static PKIXCertPathValidatorResult validate(PKIX.ValidatorParams validatorParams) throws CertPathValidatorException {
        if (PKIXCertPathValidator.debug != null) {
            PKIXCertPathValidator.debug.println("PKIXCertPathValidator.engineValidate()...");
        }
        AdaptableX509CertSelector adaptableX509CertSelector = null;
        List<X509Certificate> certificates = validatorParams.certificates();
        if (!certificates.isEmpty()) {
            adaptableX509CertSelector = new AdaptableX509CertSelector();
            X509Certificate cert = (X509Certificate)certificates.get(0);
            adaptableX509CertSelector.setSubject(cert.getIssuerX500Principal());
            try {
                adaptableX509CertSelector.setSkiAndSerialNumber(X509CertImpl.toImpl(cert).getAuthorityKeyIdentifierExtension());
            }
            catch (CertificateException | IOException ex) {}
        }
        CertPathValidatorException ex2 = null;
        for (TrustAnchor trustAnchor : validatorParams.trustAnchors()) {
            X509Certificate trustedCert = trustAnchor.getTrustedCert();
            if (trustedCert != null) {
                if (adaptableX509CertSelector != null && !adaptableX509CertSelector.match(trustedCert)) {
                    if (PKIXCertPathValidator.debug == null) {
                        continue;
                    }
                    PKIXCertPathValidator.debug.println("NO - don't try this trustedCert");
                    continue;
                }
                else if (PKIXCertPathValidator.debug != null) {
                    PKIXCertPathValidator.debug.println("YES - try this trustedCert");
                    PKIXCertPathValidator.debug.println("anchor.getTrustedCert().getSubjectX500Principal() = " + trustedCert.getSubjectX500Principal());
                }
            }
            else if (PKIXCertPathValidator.debug != null) {
                PKIXCertPathValidator.debug.println("PKIXCertPathValidator.engineValidate(): anchor.getTrustedCert() == null");
            }
            try {
                return validate(trustAnchor, validatorParams);
            }
            catch (CertPathValidatorException ex3) {
                ex2 = ex3;
            }
        }
        if (ex2 != null) {
            throw ex2;
        }
        throw new CertPathValidatorException("Path does not chain with any of the trust anchors", null, null, -1, PKIXReason.NO_TRUST_ANCHOR);
    }
    
    private static PKIXCertPathValidatorResult validate(TrustAnchor trustAnchor, PKIX.ValidatorParams validatorParams) throws CertPathValidatorException {
        UntrustedChecker untrustedChecker = new UntrustedChecker();
        X509Certificate trustedCert = trustAnchor.getTrustedCert();
        if (trustedCert != null) {
            untrustedChecker.check(trustedCert);
        }
        int size = validatorParams.certificates().size();
        ArrayList list = new ArrayList();
        list.add(untrustedChecker);
        list.add(new AlgorithmChecker(trustAnchor, null, validatorParams.date(), validatorParams.timestamp(), validatorParams.variant()));
        list.add(new KeyChecker(size, validatorParams.targetCertConstraints()));
        list.add(new ConstraintsChecker(size));
        PolicyChecker policyChecker = new PolicyChecker(validatorParams.initialPolicies(), size, validatorParams.explicitPolicyRequired(), validatorParams.policyMappingInhibited(), validatorParams.anyPolicyInhibited(), validatorParams.policyQualifiersRejected(), new PolicyNodeImpl(null, "2.5.29.32.0", null, false, Collections.singleton("2.5.29.32.0"), false));
        list.add(policyChecker);
        Date date;
        if ((validatorParams.variant() == "code signing" || validatorParams.variant() == "plugin code signing") && validatorParams.timestamp() != null) {
            date = validatorParams.timestamp().getTimestamp();
        }
        else {
            date = validatorParams.date();
        }
        BasicChecker o = new BasicChecker(trustAnchor, date, validatorParams.sigProvider(), false);
        list.add(o);
        int n = 0;
        List<PKIXCertPathChecker> certPathCheckers = validatorParams.certPathCheckers();
        for (PKIXCertPathChecker pkixCertPathChecker : certPathCheckers) {
            if ((pkixCertPathChecker instanceof PKIXRevocationChecker)) {
                if (n != 0) {
                    throw new CertPathValidatorException("Only one PKIXRevocationChecker can be specified");
                }
                n = 1;
                if ((pkixCertPathChecker instanceof RevocationChecker)) {
                    ((RevocationChecker)pkixCertPathChecker).init(trustAnchor, validatorParams);
                }
                else {
                    continue;
                }
            }
            else {
                continue;
            }
        }
        if (validatorParams.revocationEnabled() && n == 0) {
            list.add(new RevocationChecker(trustAnchor, validatorParams));
        }
        list.addAll(certPathCheckers);
        PKIXMasterCertPathValidator.validate(validatorParams.certPath(), validatorParams.certificates(), list);
        recordEvent(trustAnchor, validatorParams.certificates());
        return new PKIXCertPathValidatorResult(trustAnchor, policyChecker.getPolicyTree(), o.getPublicKey());
    }
    
    private static void recordEvent(TrustAnchor trustAnchor, List<X509Certificate> list) {
        if (EventHelper.isLoggingSecurity()) {
            int[] array = new int[list.size()];
            for (int i = 0; i < list.size(); ++i) {
                array[i] = ((X509Certificate)list.get(i)).hashCode();
            }
            EventHelper.logX509ValidationEvent(trustAnchor.getTrustedCert().hashCode(), array);
        }
    }
    
    static {
        debug = Debug.getInstance("certpath");
        validationCounter = new AtomicLong();
    }
}
